/**
 * This type union defines the expected types of draggable Elements
 */
export declare type DragElement = HTMLElement | SVGElement;
/**
 * This interface defines the message format for the `onStart`, `onDrag` and `onDrop` hooks of `DragOptions`.
 */
export interface DragMessage<TElement extends DragElement> {
    /**
     * The item being dragged
     */
    item: TElement;
    /**
     * The starting position of the item being dragged
     */
    itemOffset: ClientRect;
    /**
     * The current target to which the item is being dragged
     */
    target?: TElement;
    /**
     * The mouse-event that generated this message
     */
    event: MouseEvent;
    /**
     * Horizontal distance dragged (in pixels)
     */
    dx: number;
    /**
     * Vertical distance dragged (in pixels)
     */
    dy: number;
}
/**
 * This interface defines the callback signature of your `DragMessage` listeners
 */
export interface DragHook<TElement extends DragElement> {
    (message: DragMessage<TElement>): void;
}
/**
 * This interface the defines the callback signature of the item and target selectors.
 */
export interface ElementSelector<TElement extends DragElement> {
    (el: TElement): TElement | undefined;
}
/**
 * This union defines the allowed types for the `selectParent` utility function, and
 * the `makeParentSelector` factory-function, which accept either a string (a CSS selector)
 * or a predicate for manual filtering.
 */
export declare type ElementFilter = string | {
    (element: Element): boolean;
};
/**
 * This type defines the options that can be passed to `makeListener` factory-function.
 */
export interface DragOptions<TElement extends DragElement = HTMLElement> {
    /**
     * This hook is triggered once when the drag-operation starts
     */
    onStart?: DragHook<TElement>;
    /**
     * This hook is triggered repeatedly during the drag-operation
     */
    onDrag?: DragHook<TElement>;
    /**
     * This hook is triggered once when the drag-operation ends
     */
    onDrop?: DragHook<TElement>;
    /**
     * This function can be used to filter or change the item being dragged
     */
    selectItem?: ElementSelector<TElement>;
    /**
     * This function can be used to filter or change the target being dragged to
     */
    selectTarget?: ElementSelector<TElement>;
    /**
     * Minimum drag-distance threshold (in pixels) before the drag-operation starts.
     *
     * If specified, the `onStart` hook will not be triggered until the mouse has moved
     * a minimum distance from the position of the initial `mousedown` event - this
     * allows you to add your own `click` handlers.
     */
    dragThreshold?: number;
    /**
     * Minimum time (in milliseconds) before changing the target of a drag-operation.
     *
     * If specified, the `target` property of every `DragMessage` will not change unless
     * the user points at the same target for the specified period of time - this helps prevent
     * erratic targeting and may better track the user's intent when dragging over many small
     * moving targets, for example in a drag-and-drop list or tree.
     */
    deferTargeting?: number;
}
export interface TargetCoords {
    /**
     * Distance (in pixels) from the left edge of the current target element
     */
    left: number;
    /**
     * Distance (in pixels) from the top edge of the current target element
     */
    top: number;
    /**
     * Distance (in pixels) from the right edge of the current target element
     */
    right: number;
    /**
     * Distance (in pixels) from the bottom edge of the current target element
     */
    bottom: number;
    /**
     * Horizontal pointer location as a unit coordinate for the current target
     */
    x: number;
    /**
     * Vertical pointer location as a unit coordinate for the current target
     */
    y: number;
}
/**
 * Absolute position of the tracked element, as reported by the `itemPositionFrom` utility function.
 *
 * These coordinates can be directly applied to an element's `style.left` and `style.top` properties,
 * just make sure you add a `px` unit suffix.
 */
export interface ItemPosition {
    left: number;
    top: number;
}
/**
 * Builds a `mousedown` event-listener for custom drag-and-drop behavior defined
 * by a set of hooks and filters.
 */
export declare function makeListener<TElement extends DragElement = HTMLElement>(options: DragOptions<TElement>): (event: MouseEvent) => void;
/**
 * This function produces an element-selector that can be used to select
 * an item or target, by scanning for the nearest ancestor that matches
 * a given CSS selector or user-defined filter-function.
 */
export declare function makeParentSelector<TElement extends DragElement>(filter: ElementFilter): ElementSelector<TElement>;
/**
 * Utility function to select the nearest ancestor from a given element (or the
 * element itself) matching a given CSS selector or predicate.
 */
export declare function selectParent(element: Element, filter: ElementFilter): Element | undefined;
/**
 * Use this function to get the local target coordinates during a drag-operation.
 *
 * The `left`, `right`, `top` and `bottom` coordinates report the pointer's distance
 * from each target's edge, in pixels - this can be useful, for example, if you want
 * to check how close the pointer is hovering near the edges of the target.
 *
 * The `x` and `y` coordinates report the pointer's location within the target in unit
 * coordinates - this can be useful, for example, if you want to check if the pointer
 * is hovering over the top (`y < 0.5`) or bottom (`y >= 0.5`) of the target.
 */
export declare function targetCoordsFrom<TElement extends DragElement>({event, target}: DragMessage<TElement>): TargetCoords | undefined;
/**
 * Use this function to calculate the absolute position of the dragged item
 * (or a "ghost" representation of the dragged item) during a drag-operation.
 *
 * This takes into account the dragged item's offset position at the start of
 * the drag-operation, which means that these coordinates can be applied directly
 * (with a `px` unit suffix) to the `style.left` and `style.top` properties of
 * an absolutely-positioned element, e.g. from the `onDrag` hook.
 */
export declare function itemPositionFrom<TElement extends DragElement>({itemOffset, dx, dy}: DragMessage<TElement>): ItemPosition;
